@@ -1,14 +1,19 @@
 # 1-bit boolean vectors for R
-# (c) 2008-2009 Jens Oehlschgel
+# (c) 2008-2009 Jens Oehlschägel
+# (C) 2020 Brian Ripley
 # Licence: GPL2
 # Provided 'as is', use at your own risk
 
 # currently |.bit and |.bitwhich are bypassed if we ask for bit | bitwhich
 # xx explore/write Ops.bit Ops.bitwhich
-
+# xx bit_extract should be comlemented with
 
 # source("C:/mwp/eanalysis/bit/R/bit.R")
 
+# Configuration: set this to 32L or 64L and keep in sync with BITS in bit.c
+.BITS <- 32L
+
+
 #! \name{bit-package}
 #! \alias{bit-package}
 #! \alias{bit}
@@ -117,21 +122,17 @@
 #!   (but not longer) with an attribute 'n' and class 'bit'
 #! }
 #! \author{
-#! Jens Oehlschlgel <Jens.Oehlschlaegel@truecluster.com>
+#! Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
 #!
-#! Maintainer: Jens Oehlschlgel <Jens.Oehlschlaegel@truecluster.com>
+#! Maintainer: Jens Oehlschlägel <Jens.Oehlschlaegel@truecluster.com>
 #! }
 #! \note{
 #!   Currently operations on bit objects have some overhead from R-calls. Do expect speed gains for vectors
 #!   of length ~ 10000 or longer. \cr
 #!   Since this package was created for high performance purposes, only positive integer subscripts are allowed:
-#!   The '[.bit' and '[<-.bit' methods don't check whether the subscripts are positive integers in the allowed range.
 #!   All R-functions behave as expected - i.e. they do not change their arguments and create new return values.
 #!   If you want to save the time for return value memory allocation, you must use \code{\link{.Call}} directly
 #!   (see the dontrun example in \code{\link{sum.bit}}).
-#!   Note that the package has not been tested under 64 bit.
-#!   Note also that the mapping of NAs to TRUE differs from the mapping of NAs to FALSE
-#!   in \code{\link[ff]{vmode}="boolean"} in package ff (and one of the two may change in the future).
 #! }
 #! \keyword{ package }
 #! \keyword{ classes }
@@ -225,7 +226,7 @@
 #!     system.time(for(i in 1:100){
 #!        l[] <- TRUE
 #!     })/100
-#!     # replace bit by TRUE (NOTE that we recycle the assignment  
+#!     # replace bit by TRUE (NOTE that we recycle the assignment
 #!		 # value on R side == memory allocation and assignment first)
 #!     system.time(for(i in 1:100){
 #!        b[] <- TRUE
@@ -268,9 +269,6 @@
 #!   }
 #! }
 
-#was wrong because C-code uses int: .BITS <- 8L * .Machine$sizeof.pointer
-.BITS <- 32L
-
 
 
 #! \name{bit_init}
@@ -295,7 +293,7 @@
 #! \value{
 #!   NULL
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}  }
 #! \examples{
 #!   bit_done()
@@ -315,7 +313,6 @@ bit_done <- function()
   .Call("R_bit_done", PACKAGE="bit")
 
 
-
 # creator for empty bit vector
 bit <- function(length){
   length <- as.integer(length)
@@ -323,19 +320,19 @@ bit <- function(length){
     n <- length %/% .BITS + 1L
   else
     n <- length %/% .BITS
-  x <- integer(n)
+  if (.BITS==64L)
+    x <- integer(2L*n)
+  else
+    x <- integer(n)
   #physical(x) <- list(vmode="boolean")
   #virtual(x)  <- list(Length=length)
   #class(x) <- "bit"
   # tuning
   p <- list()
   v <- list()
-  #attributes(p) <- list(vmode="boolean", class="physical")
-  #attributes(v) <- list(Length=length, class="virtual")
-  #attributes(x) <- list(physical=p, virtual=v, class="bit")
-  setattributes(p, list(vmode="boolean", class="physical"))
-  setattributes(v, list(Length=length, class="virtual"))
-  setattributes(x, list(physical=p, virtual=v, class="bit"))
+  attributes(p) <- list(vmode="boolean", class="physical")
+  attributes(v) <- list(Length=length, class="virtual")
+  attributes(x) <- list(physical=p, virtual=v, class="bit")
   x
 }
 
@@ -389,7 +386,7 @@ print.bit <- function(x, ...){
 #!    \item unique negative integers to exclude those
 #!   }
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{as.bitwhich}}, \code{\link{as.which}}, \code{\link{bit}} }
 #! \examples{
 #!  bitwhich(12, x=c(1,3), poslength=2)
@@ -406,12 +403,11 @@ bitwhich <- function(maxindex, poslength=NULL, x=NULL){
   }else{
     poslength <- as.integer(poslength)
   }
-  #attr(x, "maxindex") <- as.integer(maxindex)
-  #attr(x, "poslength") <- poslength
+  attr(x, "maxindex") <- as.integer(maxindex)
+  attr(x, "poslength") <- poslength
   # NOTE: here we want one (1) copy of x to not modify argument x
-  class(x) <- "bitwhich"
-  setattr(x, "maxindex", as.integer(maxindex))
-  setattr(x, "poslength", poslength)
+	# therefore we did not replace the oldClass assignment with a call to setttattr
+  oldClass(x) <- "bitwhich"
   x
 }
 
@@ -440,7 +436,7 @@ print.bitwhich <- function(x, ...){
 #! \value{
 #!   TRUE or FALSE
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{is.logical}}, \code{\link{bit}}, \code{\link{bitwhich}} }
 #! \examples{
 #!  is.ri(TRUE)
@@ -501,7 +497,7 @@ is.bitwhich <- function(x)
 #! \value{
 #!   the length  A bit vector with the new length
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{length}}, \code{\link[=sum.bit]{sum}}, \code{\link[ff]{poslength}}, \code{\link[ff]{maxindex}} }
 #! \examples{
 #!   stopifnot(length(ri(1, 1, 32))==32)
@@ -594,35 +590,36 @@ is.bitwhich <- function(x)
 length.bit <- function(x)
   virtual(x)$Length
 
-"length<-.bit" <- function(x, value){
-  if (value!=length(x)){
+## The original design here was broken when extending vectors
+"length<-.bit" <- function(x, value) {
     value <- as.integer(value)
-    dn <- value %% .BITS
-    if (dn){
-      n <- value %/% .BITS + 1L
-      .Call("R_bit_replace", x, (value+1L):(value+dn), logical(dn), FALSE, PACKAGE="bit")
-    }else{
-      n <- value %/% .BITS
-    }
-    #pattr <- physical(x)
-    #vattr <- virtual(x)
-    pattr <- attr(x, "physical")
-    vattr <- attr(x, "virtual")
-    cl <- oldClass(x)
-    #x <- unclass(x)
-    attr(x, "class") <- NULL
-    length(x) <- n
-    #vattr$Length <- value
-    attr(vattr, "Length") <- value
-    #physical(x) <- pattr
-    #virtual(x) <- vattr
-    #class(x) <- cl
-    attr(x, "physical") <- pattr
-    attr(x, "virtual") <- vattr
-    attr(x, "class") <- cl
-    x
-  }else
-    x
+    lx <- length(x)
+    if (value != lx) {
+        if (value < lx) { # shrinking
+            if (dn <- value %% .BITS) {
+                n <- value %/% .BITS + 1L
+                .Call("R_bit_replace", x, (value+1L):(value+dn), logical(dn),
+                      PACKAGE = "bit")
+            } else {
+                n <- value %/% .BITS
+            }
+        } else { # growing
+            ## underlying vector is already padded with zeroes so just
+            ## extend if needed.
+            n <- .BITS * ceiling(value/.BITS)
+        }
+        pattr <- attr(x, "physical")
+        vattr <- attr(x, "virtual")
+        cl <- oldClass(x)
+        attr(x, "class") <- NULL
+        ## shrink or grow underlying vector: might be a noop
+        length(x) <- if (.BITS == 64L) 2L*n else n
+        attr(vattr, "Length") <- value
+        attr(x, "physical") <- pattr
+        attr(x, "virtual") <- vattr
+        attr(x, "class") <- cl
+        x
+    } else x
 }
 
 
@@ -634,8 +631,7 @@ length.bitwhich <- function(x)
     value <- as.integer(value)
     if (is.integer(x)){
       cl <- oldClass(x)
-      #oldClass(x) <- NULL
-      setattr(x, "class", NULL)
+      oldClass(x) <- NULL
       if (x[1]>0){
         x <- x[x <= value]
         l <- length(x)
@@ -645,8 +641,7 @@ length.bitwhich <- function(x)
           x <- TRUE
         else if (l>(value%/%2L))
           x <- -as.integer(seq_len(value))[-x]
-        #attr(x, "poslength") <- l
-        setattr(x, "poslength", l)
+        attr(x, "poslength") <- l
       }else{
         x <- x[x >= -value]
         l <- length(x)
@@ -656,17 +651,13 @@ length.bitwhich <- function(x)
           x <- FALSE
         else if (!((value-l)>(value%/%2L)))
           x <- -as.integer(seq_len(value))[-x]
-        #attr(x, "poslength") <- value - l
-        setattr(x, "poslength", value - l)
+        attr(x, "poslength") <- value - l
       }
-      #oldClass(x) <- cl
-      setattr(x, "class", cl)
+      oldClass(x) <- cl
     }else if(x){
-      #attr(x, "poslength") <- value
-      setattr(x, "poslength", value)
+      attr(x, "poslength") <- value
     }
-    #attr(x, "maxindex") <- value
-    setattr(x, "maxindex", value)
+    attr(x, "maxindex") <- value
   }
   x
 }
@@ -693,7 +684,7 @@ length.bitwhich <- function(x)
 #! \value{
 #!   An object of class 'bit'
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{c}}, \code{\link{bit}} , \code{\link{bitwhich}} }
 #! \examples{
 #!  c(bit(4), bit(4))
@@ -763,7 +754,7 @@ c.bitwhich <- function(...){
 #! \value{
 #!   \code{is.bit} returns FALSE or TRUE, \code{as.bit} returns a vector of class 'bit'
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}, \code{\link[bit:as.logical.bit]{as.logical}} }
 #! \examples{
 #!   x <- as.bit(c(FALSE, NA, TRUE))
@@ -800,7 +791,7 @@ as.bit.bitwhich <- function(x, ...){
     if (p==n)
       b[] <- TRUE
   }else{
-    class(x) <- NULL
+    oldClass(x) <- NULL
     x <- as.integer(x)
     if (x[1]<0){
       b[-x] <- TRUE  # remember that negative indices are not allowed (and the assignment value is recycled to the length of the index)
@@ -868,7 +859,7 @@ as.bit.ri <- function(x, ...){
 #! \value{
 #!   \code{\link{as.logical}} returns a vector of \code{FALSE, TRUE}, \code{\link{as.integer}} and \code{\link{as.double}} return a vector of \code{0,1}.
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{as.bit}}, \code{\link{as.which}}, \code{\link{as.bitwhich}}, \code{\link[ff]{as.ff}}, \code{\link[ff]{as.hi}} }
 #! \examples{
 #!   x <- ri(2, 5, 10)
@@ -961,7 +952,7 @@ as.double.ri <- function(x, ...){
 #! \value{
 #!   a vector of class 'logical' or 'integer'
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{as.bit}}, \code{\link{as.logical}}, \code{\link{as.integer}}, \code{\link{as.which}}, \code{\link{as.bitwhich}}, \code{\link[ff]{as.ff}}, \code{\link[ff]{as.hi}} }
 #! \examples{
 #!   r <- ri(5, 20, 100)
@@ -979,15 +970,13 @@ as.double.ri <- function(x, ...){
 
 as.which.default <- function(x, ...){
   ret <- which(x)
-  #class(ret) <- "which"
-  setattr(ret, "class", "which")
+  oldClass(ret) <- "which"
   ret
 }
 
 as.which.ri <- function(x, ...){
   ret <- x[1]:x[2]
-  #class(ret) <- "which"
-  setattr(ret, "class", "which")
+  oldClass(ret) <- "which"
   ret
 }
 
@@ -1007,8 +996,7 @@ as.which.bit <- function(x, range=NULL, ...){
     x <- as.integer(seq.int(from=range[1], to=range[2], by=1))
   }else
     x <- .Call("R_bit_which", x, s, range, negative=FALSE, PACKAGE="bit")
-  #class(x) <- "which"
-  setattr(x, "class", "which")
+  oldClass(x) <- "which"
   x
 }
 
@@ -1025,8 +1013,7 @@ as.which.bitwhich <- function(x, ...){
       attributes(x) <- NULL
     }
   }
-  #class(x) <- "which"
-  setattr(x, "class", "which")
+  oldClass(x) <- "which"
   x
 }
 
@@ -1064,7 +1051,7 @@ as.which.bitwhich <- function(x, ...){
 #! \value{
 #!   a value of class \code{\link{bitwhich}}
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bitwhich}}, \code{\link{as.bit}} }
 #! \examples{
 #!  as.bitwhich(c(FALSE, FALSE, FALSE))
@@ -1284,7 +1271,7 @@ as.logical.bitwhich <- function(x, ...){
 #! \value{
 #!   An object of class 'bit' (or 'bitwhich')
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}, \code{\link{Logic}} }
 #! \examples{
 #!   x <- as.bit(c(FALSE, FALSE, FALSE, NA, NA, NA, TRUE, TRUE, TRUE))
@@ -1587,7 +1574,7 @@ xor.bitwhich(e1, e2)
 #! \value{
 #!   as expected
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}, \code{\link{all}}, \code{\link{any}}, \code{\link{min}}, \code{\link{max}}, \code{\link{range}}, \code{\link{sum}}, \code{\link{summary}} }
 #! \examples{
 #!   x <- as.bit(c(TRUE, TRUE))
@@ -1631,7 +1618,7 @@ xor.bitwhich(e1, e2)
 #!       s <- 0L
 #!       l <- logical(N)
 #!       for (b in 1:B){
-#!         .Call("R_bit_extract", x, ((b-1L)*N+1L):(b*N), l, PACKAGE = "bit")
+#!         .Call("R_bit_extract", x, length(x), ((b-1L)*N+1L):(b*N), l, PACKAGE = "bit")
 #!         s <- s + sum(l)
 #!       }
 #!       if (R)
@@ -1902,7 +1889,7 @@ if (FALSE){
 #!   The extractors \code{[[} and \code{[} return a logical scalar or vector.
 #!   The replacment functions return a bit object.
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}, \code{\link{Extract}} }
 #! \examples{
 #!   x <- as.bit(c(FALSE, NA, TRUE))
@@ -1929,8 +1916,8 @@ if (FALSE){
     if (is.na(i) || i<1L || i>length(x))
       stop("subscript must be positive integer (or double) within length")
     ret <- logical(1L)
-    setattr(ret, "vmode", "boolean")
-    .Call("R_bit_extract", x, i, ret, PACKAGE="bit")
+    attr(ret, "vmode") <- "boolean"
+    .Call("R_bit_extract", x, length(x), i, ret, PACKAGE="bit")
   }else
     stop("subscript must be positive integer (or double) within length")
 }
@@ -1943,10 +1930,12 @@ if (FALSE){
     stop("value length not 1")
   if (is.numeric(i)){
     i <- as.integer(i)
-    if (is.na(i) || i<1L || i>length(x))
-      stop("subscript must be positive integer (or double) within length")
+    if (is.na(i) || i<1L)
+      stop("subscript must be positive integer (or double)")
+    if ((mi <- max(i))>length(x))
+      length(x) <- mi
     value2 <- as.logical(value)
-    .Call("R_bit_replace", x, i, value2, FALSE, PACKAGE="bit")
+    .Call("R_bit_replace", x, i, value2, PACKAGE="bit")
   }else
     stop("subscript must be positive integer (or double) within length")
 }
@@ -1972,66 +1961,66 @@ if (FALSE){
 
 
 "[.bit" <- function(x, i){
+  nx <- length(x)
   if ( missing(i) ){
-    len <- length(x)
-    ret <- logical(len)
-    .Call("R_bit_get", x, ret, range=c(1L, len), PACKAGE="bit")
+    ret <- logical(nx)
+    .Call("R_bit_get", x, ret, range=c(1L, nx), PACKAGE="bit")
   }else if(is.numeric(i)){
     if (inherits(i, "ri")){
-      if (i[1]<1L || i[2]>length(x))
+      if (i[1]<1L || i[2]>nx )
         stop("illegal range index 'ri'")
       ret <- logical(i[2]-i[1]+1L)
       .Call("R_bit_get", x, ret, range=i, PACKAGE="bit")
     }else{
-      i <- as.integer(i)
-      n <- length(i)
-      N <- length(x)
-      if (n==0){
-        ret <- logical()
+      if (inherits(i, "bitwhich")){
+        i <- as.which(i)
+        n <- length(i)
       }else{
-        if (i[1]<0){
-          i <- (as.integer(seq_len(N)))[i]
-          ret <- logical(N-n)
-        }else{
-          ret <- logical(length(i))
+        i <- as.integer(i)
+        n <- length(i)
+        if (n && i[1]<0){
+          i <- (as.integer(seq_along(x)))[i]
+          n <- length(i)
         }
-        .Call("R_bit_extract", x, i, ret, PACKAGE="bit")
       }
+			ret <- logical(n)
+			if (n)
+        .Call("R_bit_extract", x, nx, i, ret, PACKAGE="bit")
     }
   }else if(is.logical(i)){
     if (length(i)!=1 || is.na(i)){
-      stop("only TRUE or FALSE allowed")
+      stop("only scalar TRUE or FALSE allowed")
     }else{
       if (i){
-        len <- length(x)
-        ret <- logical(len)
-        .Call("R_bit_get", x, ret, range=c(1L, len), PACKAGE="bit")
+        ret <- logical(nx)
+        .Call("R_bit_get", x, ret, range=c(1L, nx), PACKAGE="bit")
       }else{
         ret <- logical()
       }
     }
   }else
-      stop("subscript must be integer (or double) or bitwhich")
-
-  setattr(ret, "vmode", "boolean")
+      stop("subscript must be integer (or double) or bitwhich or TRUE or FALSE")
+  attr(ret, "vmode") <- "boolean"
   ret
 }
 
 
 "[<-.bit" <- function(x, i, value){
+  nx <- length(x)
   if ( missing(i) ){
-    len <- length(x)
-    if (length(value)==len){
+    if (length(value)==nx){
       value2 <- as.logical(value)
     }else{
-      value2 <- logical(len)
+      value2 <- logical(nx)
       value2[] <- value
     }
-    .Call("R_bit_set", x, value2, range=c(1L, len), PACKAGE="bit")
+    .Call("R_bit_set", x, value2, range=c(1L, nx), PACKAGE="bit")
   }else if(is.numeric(i)){
     if (inherits(i, "ri")){
-      if (i[1]<1L || i[2]>length(x))
+      if (i[1]<1L)
         stop("illegal range index 'ri'")
+      if (i[2]>nx)
+        length(x) <- i[2]
       n <- i[2] - i[1] + 1L
       if (length(value)==n){
         value2 <- as.logical(value)
@@ -2041,42 +2030,45 @@ if (FALSE){
       }
       .Call("R_bit_set", x, value2, range=i, PACKAGE="bit")
     }else{
-      i <- as.integer(i)
-      n <- length(i)
-      N <- length(x)
-      if (n==0)
-        return(x)
-      if (i[1]<0){
-        i <- (as.integer(seq_len(N)))[i]
-        n <- N - n
-      }
+			if (inherits(i, "bitwhich")){
+				i <- as.which(i)
+				n <- length(i)
+			}else{
+				i <- as.integer(i)
+				n <- length(i)
+				if (n && i[1]<0){
+					i <- (as.integer(seq_along(x)))[i]
+					n <- length(i)
+				}
+			}
+      if ((mi <- max(i))>nx)
+        length(x) <- mi
       if (length(value)==n){
         value2 <- as.logical(value)
       }else{
-        value2 <- logical(length(i))
+        value2 <- logical(n)
         value2[] <- value
       }
       .Call("R_bit_replace", x, i, value2, PACKAGE="bit")
     }
   }else if (is.logical(i)){
     if (length(i)!=1 || is.na(i)){
-      stop("only TRUE or FALSE allowed")
+      stop("only scalar TRUE or FALSE allowed")
     }else{
       if (i){
-        len <- length(x)
-        if (length(value)==len){
+        if (length(value)==nx){
           value2 <- as.logical(value)
         }else{
-          value2 <- logical(len)
+          value2 <- logical(nx)
           value2[] <- value
         }
-        .Call("R_bit_set", x, value2, range=c(1L, len), PACKAGE="bit")
+        .Call("R_bit_set", x, value2, range=c(1L, nx), PACKAGE="bit")
       }else{
         x
       }
     }
   }else
-      stop("subscript must be integer (or double) within length")
+      stop("subscript must be integer (or double) or bitwhich or TRUE or FALSE")
 }
 
 
@@ -2103,7 +2095,7 @@ if (FALSE){
 #! \value{
 #!   A two element integer vector with class 'ri'
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link[ff]{as.hi.ri}} }
 #! \examples{
 #!  bit(12)[ri(1,6)]
@@ -2126,8 +2118,7 @@ ri <- function(from, to=NULL, maxindex=NA){
     stop("lower bound must be smaller or equal than upper bound")
   if (!is.na(x[[3]]) && x[[2]]>x[[3]])
     stop("lower and upper bound must be smaller or equal to maxindex")
-  #class(x) <- "ri"
-  setattr(x, "class", "ri")
+  oldClass(x) <- "ri"
   x
 }
 
@@ -2224,7 +2215,7 @@ summary.ri <- function(object, ...){
 #! \value{
 #!   \command{physical} and \command{virtual} returns a list with named elements
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{
 #!  \code{\link[ff]{physical.ff}}, \code{\link[ff]{physical.ffdf}}
 #! }
@@ -2244,10 +2235,7 @@ physical.default <- function(x){
   p
 }
 "physical<-.default" <- function(x, value){
-  #attributes(attr(x, "physical")) <- c(value, list(class="physical"))
-  p <- list()
-  setattributes(p, c(value, list(class="physical")))
-  setattr(x, "physical", p)
+  attributes(attr(x, "physical")) <- c(value, list(class="physical"))
   x
 }
 
@@ -2257,10 +2245,7 @@ virtual.default <- function(x){
   v[is.na(match(names(v), "class"))]
 }
 "virtual<-.default" <- function(x, value){
-  #attributes(attr(x, "virtual")) <- c(value, list(class="virtual"))
-  v <- list()
-  setattributes(v, c(value, list(class="virtual")))
-  setattr(x, "virtual", v)
+  attributes(attr(x, "virtual")) <- c(value, list(class="virtual"))
   x
 }
 
@@ -2300,7 +2285,7 @@ R_bit_as_hi <- function(x, range, offset)
 #! \value{
 #!   a vector of class 'logical' or 'integer'
 #! }
-#! \author{ Jens Oehlschlgel }
+#! \author{ Jens Oehlschlägel }
 #! \seealso{ \code{\link{bit}}, \code{\link{as.bit}}, \code{\link{as.logical}}, \code{\link{as.integer}}, \code{\link{which}} }
 #! \examples{
 #!   if (regtest.bit()){
@@ -2320,9 +2305,75 @@ regtest.bit <- function(
     N = 100  # number of repetitions for random regression tests
 )
 {
+  #.BITS <- bit:::.BITS  # available in package namespace
   OK <- TRUE
   pool <- c(FALSE, TRUE)
 
+  if (!identical(unattr(as.bit(c(FALSE,NA,TRUE))[]), c(FALSE,FALSE,TRUE))){
+    message("bit error: wrong coercion of triboolean to (bi)boolean")
+    OK <- FALSE
+  }
+
+  l <- TRUE
+  b <- as.bit(l)
+
+  i <- -c(1, 0, 1, NA)
+  if (!inherits(try(b[i], silent=TRUE), "try-error")){
+    message("bit error: did not throw on mixing zero with negative subscripts")
+    OK <- FALSE
+  }
+
+  i <- c(2, 1, 0, 1, NA)
+  if (!identical(l[i],unattr(b[i]))){
+    message("\nregression test difference between b[i] and l[i]")
+    print(l[i])
+    print(unattr(b[i]))
+    OK <- FALSE
+  }
+
+  l[0] <- TRUE
+  b[0] <- TRUE
+  if (!identical(l,unattr(b[]))){
+    message("\nregression test difference after assigning at R position zero")
+    print(l)
+    print(unattr(b[]))
+    OK <- FALSE
+ }
+
+  l[2] <- TRUE
+  b[2] <- TRUE
+  if (!identical(ifelse(is.na(l), FALSE, l),unattr(b[]))){
+    message("\nregression test difference after assigning after vector length (at 2)")
+    print(l)
+    print(unattr(b[]))
+    OK <- FALSE
+  }
+
+  l[.BITS+1] <- FALSE
+  b[.BITS+1] <- NA
+  if (!identical(ifelse(is.na(l), FALSE, l),unattr(b[]))){
+    message("\nregression test difference after assigning after vector length (at .BITS+1)")
+    print(l)
+    print(unattr(b[]))
+    OK <- FALSE
+  }
+
+  if (!identical(ifelse(is.na(l[TRUE]), FALSE, l[TRUE]),unattr(b[TRUE]))){
+    message("\nregression test difference after subscripting with scalar TRUE")
+    print(l)
+    print(unattr(b[]))
+    OK <- FALSE
+  }
+
+  if (!identical(ifelse(is.na(l[FALSE]), FALSE, l[FALSE]),unattr(b[FALSE]))){
+    message("\nregression test difference after subscripting with scalar FALSE")
+    print(l)
+    print(unattr(b[]))
+    OK <- FALSE
+  }
+
+
+
   for (i in 1:N){
     n <- sample(1:(2*.BITS), 1)
     l <- sample(pool, n, TRUE)
@@ -2336,6 +2387,8 @@ regtest.bit <- function(
       print(l2)
       OK <- FALSE
     }
+
+
     # summary functions with logical return
     s <- c(all=all(l), any=any(l))
     s2 <- c(all=all(b), any=any(b))
