@@ -1,6 +1,6 @@
 /*
 # C-Code for chunk utilities
-# (c) 2012 Jens Oehlschgel
+# (c) 2012 Jens Oehlschägel
 # Licence: GPL2
 # Provided 'as is', use at your own risk
 # Created: 2012-12-02
@@ -15,7 +15,7 @@ SEXP R_bit_vecseq(SEXP x_, SEXP y_)
 {
     int *x,*y,*ret;
     register int val, lim;
-    R_len_t K,k,n,i,j;
+    R_len_t K,k,n,i;
     SEXP ret_;
     // if (!isInteger(x_))
       // error("x must be an integer vector");
